import { jsxs, jsx } from 'react/jsx-runtime';
import { useState, useEffect, memo, useRef, useCallback } from 'react';
import styled from 'styled-components';
import { BoldOutlined, ItalicOutlined, StrikethroughOutlined, FontSizeOutlined, UnorderedListOutlined, OrderedListOutlined, LinkOutlined, PictureOutlined, TableOutlined, CodeOutlined, BulbOutlined, CompressOutlined, ExpandOutlined } from '@ant-design/icons';
import { Menu, message, Tooltip, Dropdown, Spin } from 'antd';
import MarkdownIt from 'markdown-it';
import hljs from 'highlight.js';
import 'antd/dist/antd.css';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __makeTemplateObject(cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
}

var MarkdownEditContainer = styled.div(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n    box-sizing: border-box;\n    height: 100%;\n    div, p{\n        box-sizing: border-box;\n    }\n    .spining{\n        height: calc( 100% - 50px );\n    }\n    .markdown-main{\n        height: 100%;\n        display: flex;\n        padding: 0;\n        .line-container{\n            width: 48px;\n            height: 100%;\n            overflow: hidden;\n            background: #272727;\n            padding-top: 0px;\n            box-sizing: border-box;\n            display: none;\n            .line-num{\n                width: 100%;\n                text-align: center;\n                line-height: 26px;\n                color: #fff;\n                margin: 0;\n            }\n        }\n        .markdown-editor{\n            width: calc( 50% );\n            height: 100%;\n            border: none;\n            outline: none;\n            background: #2d2d2d;\n            box-sizing: border-box;\n            resize: none;\n            font-size: 16px;\n            padding: 0 20px 0;\n            line-height: 26px;\n            color: #69c0ff;\n            transition: width .5s;\n            &.hide{\n                width: 0;\n                padding: 0;\n            }\n        }\n        .markdown-preview{\n            width: 50%;\n            height: 100%;\n            padding: 20px;\n            box-sizing: border-box;\n            overflow-y: scroll;\n            transition: width .5s;\n            &.prev{\n                width: 100%;\n            }\n        }\n    }\n"], ["\n    box-sizing: border-box;\n    height: 100%;\n    div, p{\n        box-sizing: border-box;\n    }\n    .spining{\n        height: calc( 100% - 50px );\n    }\n    .markdown-main{\n        height: 100%;\n        display: flex;\n        padding: 0;\n        .line-container{\n            width: 48px;\n            height: 100%;\n            overflow: hidden;\n            background: #272727;\n            padding-top: 0px;\n            box-sizing: border-box;\n            display: none;\n            .line-num{\n                width: 100%;\n                text-align: center;\n                line-height: 26px;\n                color: #fff;\n                margin: 0;\n            }\n        }\n        .markdown-editor{\n            width: calc( 50% );\n            height: 100%;\n            border: none;\n            outline: none;\n            background: #2d2d2d;\n            box-sizing: border-box;\n            resize: none;\n            font-size: 16px;\n            padding: 0 20px 0;\n            line-height: 26px;\n            color: #69c0ff;\n            transition: width .5s;\n            &.hide{\n                width: 0;\n                padding: 0;\n            }\n        }\n        .markdown-preview{\n            width: 50%;\n            height: 100%;\n            padding: 20px;\n            box-sizing: border-box;\n            overflow-y: scroll;\n            transition: width .5s;\n            &.prev{\n                width: 100%;\n            }\n        }\n    }\n"])));
var NavBarContainer = styled.nav(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n    width: 100%;\n    height: 50px;\n    border-bottom: 1px solid #e1e1e1;\n    box-sizing: border-box;\n    display: flex;\n    align-items: center;\n    padding: 0 20px;\n    .item{\n        cursor: pointer;\n        margin-right: 8px;\n        padding: 5px;\n        font-size: 17px;\n        border-radius: 3px;\n\n        &:hover {\n            background-color: #eee;\n        }\n\n        &.code {\n            padding: 5px 0;\n        }\n    }\n    .active{\n        background: rgba(0, 255, 255, .5);\n        border: 1px solid rgb(0, 255, 255);\n    }\n    .right {\n        flex: 1;\n        text-align: right;\n    }\n"], ["\n    width: 100%;\n    height: 50px;\n    border-bottom: 1px solid #e1e1e1;\n    box-sizing: border-box;\n    display: flex;\n    align-items: center;\n    padding: 0 20px;\n    .item{\n        cursor: pointer;\n        margin-right: 8px;\n        padding: 5px;\n        font-size: 17px;\n        border-radius: 3px;\n\n        &:hover {\n            background-color: #eee;\n        }\n\n        &.code {\n            padding: 5px 0;\n        }\n    }\n    .active{\n        background: rgba(0, 255, 255, .5);\n        border: 1px solid rgb(0, 255, 255);\n    }\n    .right {\n        flex: 1;\n        text-align: right;\n    }\n"])));
var templateObject_1, templateObject_2;

/**
 * 获取输入框选中文本的开始与结束的下标
 * @param {HTMLTextAreaElement} el
 * @returns {number[]}
 */
var getCursorPosition = function (el) {
    var selectionStart = el.selectionStart, selectionEnd = el.selectionEnd;
    return [selectionStart, selectionEnd];
};
/**
 * 处理尾部多余的换行符
 * @param {string} str
 * @returns {string}
 */
var clearEndNullText = function (str) {
    var arr = str.split('\n').reverse();
    var index = arr.findIndex(function (item) { return item.length > 0; }) - 1;
    arr.splice(0, index);
    return arr.reverse().join('\n');
};
/**
 * 获取焦点，并设置选中的文本
 * @param {HTMLTextAreaElement} el
 * @param {number} selectionStart
 * @param {number} selectionEnd
 */
var setSelectionRange = function (el, selectionStart, selectionEnd) {
    var timer = setTimeout(function () {
        var scrollTop = el.scrollTop;
        el.focus();
        el.scrollTop = scrollTop;
        el.setSelectionRange(selectionStart, selectionEnd, 'none');
        timer && clearInterval(timer);
    }, 0);
};
/**
 * 添加加粗、斜体、中划线的方法
 * @param {HTMLTextAreaElement} el
 * @param {string} symbol
 * @param {string} txt
 * @param {Function} setValue
 */
var handleText = function (el, symbol, txt, setValue) {
    var _a = getCursorPosition(el), start = _a[0], end = _a[1];
    var value = start === end
        ? el.value.slice(0, start) + "\n" + symbol + txt + symbol + "\n" + el.value.slice(end)
        : "" + el.value.slice(0, start) + symbol + el.value.slice(start, end) + symbol + el.value.slice(end);
    var selectionStart = start === end ? start + symbol.length + 1 : start + symbol.length;
    var selectionEnd = start === end ? selectionStart + txt.length : end + symbol.length;
    value = clearEndNullText(value);
    setSelectionRange(el, selectionStart, selectionEnd);
    setValue(value, selectionStart, selectionEnd);
};
/**
 * 添加标题
 * @param {HTMLTextAreaElement} el
 * @param {string} symbol
 * @param {string} txt
 * @param {Function} setValue
 */
var addTitle = function (el, symbol, txt, setValue) {
    var _a = getCursorPosition(el), start = _a[0], end = _a[1];
    var value = start === end
        ? el.value.slice(0, start) + "\n" + symbol + " " + txt + "\n" + el.value.slice(end)
        : el.value.slice(0, start) + "\n" + symbol + " " + el.value.slice(start, end) + "\n" + el.value.slice(end);
    var selectionStart = start + symbol.length + 2;
    var selectionEnd = start === end ? selectionStart + txt.length : end + symbol.length + 1;
    value = clearEndNullText(value);
    setSelectionRange(el, selectionStart, selectionEnd);
    setValue(value, selectionStart, selectionEnd);
};
/**
 * 添加有序列表、无序列表, tab缩进
 * @param {HTMLTextAreaElement} el
 * @param {string} symbol
 * @param {string} txt
 * @param {Function} setValue
 * @param {1 | 2} type  1: 有序、无序列表  2: 其他，如tab缩进
 */
var addList = function (el, symbol, setValue, type) {
    if (type === void 0) { type = 1; }
    var _a = getCursorPosition(el), start = _a[0], end = _a[1];
    var paragraph = el.value.split('\n'), activeStart = start, flag = start === end, value = "", selectionStart = start, selectionEnd = end, len = paragraph.length, stringCount = 0, addSpaceCount = 0;
    activeStart = el.value.slice(0, start).lastIndexOf('\n') === -1 ? 0 : el.value.slice(0, start).lastIndexOf('\n');
    activeStart = activeStart === 0 ? 0 : activeStart + 1;
    if (flag) {
        if (type === 1) {
            value = "" + el.value.slice(0, activeStart) + symbol + el.value.slice(activeStart);
        }
        else if (type === 2) {
            value = "" + el.value.slice(0, start) + symbol + el.value.slice(end);
        }
        selectionStart += symbol.length;
        selectionEnd += symbol.length;
    }
    else {
        for (var i = 0; i < len; i++) {
            var item = paragraph[i], nextStringCount = stringCount + item.length + 1;
            if (nextStringCount > start && stringCount < end) {
                var newItem = "" + symbol + item;
                addSpaceCount += symbol.length;
                paragraph[i] = newItem;
                if (start > stringCount)
                    selectionStart += symbol.length;
                if (nextStringCount > end)
                    selectionEnd += addSpaceCount;
            }
            else if (stringCount > end) {
                break;
            }
            stringCount = nextStringCount;
        }
        value = paragraph.join('\n');
    }
    value = clearEndNullText(value);
    setSelectionRange(el, selectionStart, selectionEnd);
    setValue(value, selectionStart, selectionEnd);
};
/**
 * 添加超链接
 * @param {HTMLTextAreaElement} el
 * @param {Function} setValue
 */
var addLink = function (el, setValue) {
    var _a = getCursorPosition(el), start = _a[0], end = _a[1];
    var value = start === end
        ? el.value.slice(0, start) + "[\u94FE\u63A5\u6587\u5B57](url)" + el.value.slice(end)
        : el.value.slice(0, start) + "[" + el.value.slice(start, end) + "](url)" + el.value.slice(end);
    var selectionStart = start === end ? start + 7 : end + 3;
    var selectionEnd = start === end ? end + 10 : end + 6;
    value = clearEndNullText(value);
    setSelectionRange(el, selectionStart, selectionEnd);
    setValue(value, selectionStart, selectionEnd);
};
/**
 * 添加图片
 * @param {HTMLTextAreaElement} el
 * @param {Function} setValue
 */
var addPhoto = function (el, setValue) {
    var _a = getCursorPosition(el), start = _a[0], end = _a[1];
    var value = start === end
        ? el.value.slice(0, start) + "\n![image](url)\n" + el.value.slice(end)
        : el.value.slice(0, start) + "\n![" + el.value.slice(start, end) + "](url)\n" + el.value.slice(end);
    var selectionStart = start === end ? start + 10 : end + 5;
    var selectionEnd = start === end ? end + 13 : end + 8;
    value = clearEndNullText(value);
    setSelectionRange(el, selectionStart, selectionEnd);
    setValue(value, selectionStart, selectionEnd);
};
/**
 * 添加表格
 * @param {HTMLTextAreaElement} el
 * @param {Function} setValue
 * @param {number} row
 * @param {number} col
 */
var addTable = function (el, setValue, row, col) {
    if (row === void 0) { row = 2; }
    if (col === void 0) { col = 3; }
    var _a = getCursorPosition(el), start = _a[0], end = _a[1];
    var tableStr = "";
    for (var i = 0; i < row; i++) {
        for (var j = 0; j < col; j++) {
            var value_1 = (i === 0 && j === 0) ? (start === end ? i + 1 + "-" + (j + 1) : "" + el.value.slice(start, end)) : i + 1 + "-" + (j + 1);
            tableStr += j === col - 1 ? "|" + value_1 + "|" : "|" + value_1;
        }
        tableStr += "\n";
        if (i === 0) {
            for (var j = 0; j < col; j++) {
                tableStr += j === col - 1 ? "| -- |" : "| -- ";
            }
            tableStr += "\n";
        }
    }
    var value = el.value.slice(0, start) + "\n" + tableStr + "\n" + el.value.slice(end);
    var selectionStart = start + 2;
    var selectionEnd = start === end ? selectionStart + 3 : selectionStart + end - start;
    value = clearEndNullText(value);
    setSelectionRange(el, selectionStart, selectionEnd);
    setValue(value, selectionStart, selectionEnd);
};
/**
 * 添加代码块
 * @param {HTMLTextAreaElement} el
 * @param {Function} setValue
 */
var addCode = function (el, setValue) {
    var _a = getCursorPosition(el), start = _a[0], end = _a[1];
    var value = start === end
        ? el.value.slice(0, start) + "\n```\n\n```\n" + el.value.slice(end)
        : el.value.slice(0, start) + "\n```\n" + el.value.slice(start, end) + "\n```\n" + el.value.slice(end);
    var selectionStart = start + 5;
    var selectionEnd = selectionStart;
    value = clearEndNullText(value);
    setSelectionRange(el, selectionStart, selectionEnd);
    setValue(value, selectionStart, selectionEnd);
};
/**
 * shift + tab 清除段落最开始的空字符
 * @param {HTMLTextAreaElement} el
 * @param {number} tabSpaceCount
 * @param {Function} setValue
 */
var cancelTabSpace = function (el, tabSpaceCount, setValue) {
    var _a = getCursorPosition(el), start = _a[0], end = _a[1];
    var paragraph = el.value.split('\n'), selectionStart = start, selectionEnd = end, value = "", stringCount = 0, cancelSpaceCount = 0, len = paragraph.length;
    for (var i = 0; i < len; i++) {
        var item = paragraph[i];
        var nextStringCount = stringCount + item.length + 1;
        if (nextStringCount > start && stringCount < end) {
            var spaces = item.split(' '.repeat(tabSpaceCount));
            if (spaces.length !== 1 && spaces[0] === "") {
                spaces.shift();
                cancelSpaceCount += tabSpaceCount;
            }
            else {
                var oldlen = spaces[0].length;
                spaces[0] = spaces[0].trimLeft();
                var newlen = spaces[0].length;
                cancelSpaceCount += (oldlen - newlen);
            }
            var newParagraph = spaces.join(' '.repeat(tabSpaceCount));
            paragraph[i] = newParagraph;
            if (start > stringCount)
                selectionStart -= item.length - newParagraph.length;
            if (end < nextStringCount)
                selectionEnd -= cancelSpaceCount;
        }
        else if (stringCount > end) {
            break;
        }
        stringCount = nextStringCount;
    }
    value = paragraph.join('\n');
    setSelectionRange(el, selectionStart, selectionEnd);
    setValue(value, selectionStart, selectionEnd);
};
/**
 * 重写按下回车执行的操作
 * @param {HTMLTextAreaElement} el
 * @param {Function} setValue
 * @param {number} tabSpace
 */
var clickEnter = function (el, setValue, tabSpace) {
    setTimeout(function () {
        var _a = getCursorPosition(el), start = _a[0], end = _a[1];
        var flag = start === end, value, selectionStart = start, selectionEnd = end;
        if (flag) {
            var activeStr = el.value[start - 1];
            var spaceStr = getActiveLineBeginSpaces(el, tabSpace);
            if (activeStr === '{') {
                if (el.value[start] === '}') {
                    value = el.value.slice(0, start) + "\n" + (' '.repeat(tabSpace) + spaceStr) + "\n" + spaceStr + el.value.slice(end);
                    selectionStart = start + 1 + tabSpace + spaceStr.length;
                    selectionEnd = end + 1 + tabSpace + spaceStr.length;
                }
                else {
                    value = el.value.slice(0, start) + "\n" + (' '.repeat(tabSpace) + spaceStr) + "\n" + (el.value.slice(end).trimLeft()[0] === '}' ? el.value.slice(end) : spaceStr + (el.value.slice(end)));
                    selectionStart = start + 1 + tabSpace + spaceStr.length;
                    selectionEnd = end + 1 + tabSpace + spaceStr.length;
                }
            }
            else {
                value = el.value.slice(0, start) + "\n" + spaceStr + el.value.slice(end);
                selectionStart = start + 1 + spaceStr.length;
                selectionEnd = end + 1 + spaceStr.length;
            }
        }
        else {
            value = el.value.slice(0, start) + "\n" + el.value.slice(end);
            selectionStart = start + 1;
            selectionEnd = start + 1;
        }
        setSelectionRange(el, selectionStart, selectionEnd);
        setValue(value, selectionStart, selectionEnd);
    }, 0);
};
/**
 * 自动补全括号、引号 () [] {} '' ""
 * @param {HTMLTextAreaElement} el
 * @param {Function} setValue
 * @param {string[]} bracket
 */
var autoComplementBracket = function (el, setValue, bracket) {
    var _a = getCursorPosition(el), start = _a[0], end = _a[1];
    var flag = start === end, value = el.value, selectionStart = start, selectionEnd = end;
    if (flag) {
        value = "" + el.value.slice(0, start) + bracket.join('') + el.value.slice(end);
        selectionStart = start + bracket[0].length;
        selectionEnd = end + bracket[0].length;
    }
    else {
        value = "" + el.value.slice(0, start) + bracket[0] + el.value.slice(start, end) + bracket[1] + el.value.slice(end);
        selectionStart = start + bracket[0].length;
        selectionEnd = end + bracket[0].length;
    }
    setSelectionRange(el, selectionStart, selectionEnd);
    setValue(value, selectionStart, selectionEnd);
};
/**
 * 获取光标所在行前面的tab字符
 * @param {HTMLTextAreaElement} el
 * @param {number} tabSpace
 * @returns {string}
 */
var getActiveLineBeginSpaces = function (el, tabSpace) {
    var _a = getCursorPosition(el), start = _a[0], end = _a[1];
    var flag = start === end, stringCount = 0, nextStringCount = 0, paragraph = el.value.split('\n'), spaces = 0;
    if (flag) {
        var len = paragraph.length;
        for (var i = 0; i < len; i++) {
            stringCount += (i === 0 ? 0 : (paragraph[i - 1].length + 1));
            nextStringCount = stringCount + paragraph[i].length + 1;
            if (stringCount < start && nextStringCount > start) {
                var items = paragraph[i].split(' '.repeat(tabSpace));
                for (var j = 0; j < items.length; j++) {
                    if (items[j] === "") {
                        spaces++;
                    }
                    else {
                        break;
                    }
                }
                if (items[items.length - 1] === "") {
                    spaces--;
                }
                break;
            }
        }
    }
    return spaces === 0 ? '' : ' '.repeat(spaces * tabSpace);
};

var Item = Menu.Item, ItemGroup = Menu.ItemGroup;
var codeThemeList = ['default', 'hybrid', 'dark', 'github', 'atelier-lakeside-dark', 'color-brewer', 'docco', 'mono-blue', 'paraiso-dark'];
// const markdownThemeList = ['maize', 'guthub']
var style;
var NavBar = function (_a) {
    var setValue = _a.setValue, editorElement = _a.editorElement, setLoading = _a.setLoading, preview = _a.preview, setPreview = _a.setPreview;
    var _b = useState('hybrid'), codeTheme = _b[0], setCodeTheme = _b[1];
    useEffect(function () {
        var head = document.head;
        var oldLink = head.getElementsByClassName('markdownTheme-style-link');
        if (oldLink.length)
            head.removeChild(oldLink[0]);
        var newLink = document.createElement('link');
        newLink.setAttribute('rel', 'stylesheet');
        newLink.setAttribute('type', 'text/css');
        newLink.setAttribute('class', 'markdownTheme-style-link');
        newLink.setAttribute('href', "http://cpyfiles.dabanheiji.com/github-markdown.css");
        newLink.onload = function () { return setLoading(false); };
        newLink.onerror = function () {
            setLoading(false);
            message.error('主题获取失败，请稍后重试或尝试其它主题');
        };
        head.appendChild(newLink);
    }, []);
    useEffect(function () {
        setLoading(true);
        var head = document.head;
        var oldLink = head.getElementsByClassName('highlightjs-style-link');
        if (oldLink.length)
            head.removeChild(oldLink[0]);
        var newLink = document.createElement('link');
        newLink.setAttribute('rel', 'stylesheet');
        newLink.setAttribute('type', 'text/css');
        newLink.setAttribute('class', 'highlightjs-style-link');
        newLink.setAttribute('href', "https://cdn.bootcss.com/highlight.js/9.6.0/styles/" + codeTheme + ".min.css");
        newLink.onload = function () { return setLoading(false); };
        newLink.onerror = function () {
            setLoading(false);
            message.error('主题获取失败，请稍后重试或尝试其它主题');
        };
        head.appendChild(newLink);
        style && head.removeChild(style);
        var color = "#1d1f21";
        if (codeTheme === 'default' || codeTheme === 'github' || codeTheme === 'color-brewer' || codeTheme === 'docco' || codeTheme === 'mono-blue') {
            color = "#f6f8fa";
        }
        style = document.createElement('style');
        style.textContent = "\n        .markdown-body .highlight pre, .markdown-body pre{\n            background-color: " + color + " !important;\n        }\n        .spining .ant-spin-container{\n            height: 100%;\n        }\n        .code-highlight-theme-menu .active, .markdown-theme-menu .active {\n            background-color: rgb(225, 248, 222);\n            border-radius: 3px;\n        }\n        ";
        head.appendChild(style);
    }, [codeTheme]);
    var TitleMenu = (jsxs(Menu, __assign({ onClick: function (a) { return addTitle(editorElement, a.key, a.domEvent.target.innerText, setValue); } }, { children: [jsx(Item, { children: "\u4E00\u7EA7\u6807\u9898" }, "#"),
            jsx(Item, { children: "\u4E8C\u7EA7\u6807\u9898" }, "##"),
            jsx(Item, { children: "\u4E09\u7EA7\u6807\u9898" }, "###"),
            jsx(Item, { children: "\u56DB\u7EA7\u6807\u9898" }, "####"),
            jsx(Item, { children: "\u4E94\u7EA7\u6807\u9898" }, "#####"),
            jsx(Item, { children: "\u516D\u7EA7\u6807\u9898" }, "######")] }), void 0));
    var CodeMenu = (jsx(Menu, __assign({ onClick: function (e) { return setCodeTheme(e.key); } }, { children: jsx(ItemGroup, __assign({ title: "\u4EE3\u7801\u5757\u98CE\u683C", className: "item-group-list-container code-highlight-theme-menu" }, { children: codeThemeList.map(function (item, index) {
                return (jsx(Item, __assign({ className: codeTheme === item ? "active" : '' }, { children: item }), item));
            }) }), void 0) }), void 0));
    return (jsxs(NavBarContainer, { children: [jsx(Tooltip, __assign({ title: "\u52A0\u7C97" }, { children: jsx(BoldOutlined, { className: "item", onClick: function () { return handleText(editorElement, '**', '加粗文本', setValue); } }, void 0) }), void 0),
            jsx(Tooltip, __assign({ title: "\u659C\u4F53" }, { children: jsx(ItalicOutlined, { className: "item", onClick: function () { return handleText(editorElement, '*', '斜体文本', setValue); } }, void 0) }), void 0),
            jsx(Tooltip, __assign({ title: "\u4E2D\u5212\u7EBF" }, { children: jsx(StrikethroughOutlined, { className: "item", onClick: function () { return handleText(editorElement, '~~', '中划线', setValue); } }, void 0) }), void 0),
            jsx(Dropdown, __assign({ overlay: TitleMenu, placement: "bottomCenter", arrow: true }, { children: jsx(FontSizeOutlined, { className: "item" }, void 0) }), void 0),
            jsx(Tooltip, __assign({ title: "\u65E0\u5E8F\u5217\u8868" }, { children: jsx(UnorderedListOutlined, { className: "item", onClick: function () { return addList(editorElement, '- ', setValue); } }, void 0) }), void 0),
            jsx(Tooltip, __assign({ title: "\u6709\u5E8F\u5217\u8868" }, { children: jsx(OrderedListOutlined, { className: "item", onClick: function () { return addList(editorElement, '1. ', setValue); } }, void 0) }), void 0),
            jsx(Tooltip, __assign({ title: "\u8D85\u94FE\u63A5" }, { children: jsx(LinkOutlined, { className: "item", onClick: function () { return addLink(editorElement, setValue); } }, void 0) }), void 0),
            jsx(Tooltip, __assign({ title: "\u56FE\u7247" }, { children: jsx(PictureOutlined, { className: "item", onClick: function () { return addPhoto(editorElement, setValue); } }, void 0) }), void 0),
            jsx(Tooltip, __assign({ title: "\u8868\u683C" }, { children: jsx(TableOutlined, { className: "item", onClick: function () { return addTable(editorElement, setValue); } }, void 0) }), void 0),
            jsx(Tooltip, __assign({ title: "\u4EE3\u7801\u5757" }, { children: jsx(CodeOutlined, { className: "item", onClick: function () { return addCode(editorElement, setValue); } }, void 0) }), void 0),
            jsx(Dropdown, __assign({ overlay: CodeMenu, placement: "bottomCenter", arrow: true }, { children: jsx(BulbOutlined, { className: "item" }, void 0) }), void 0),
            false ,
            jsx("div", __assign({ className: "right" }, { children: preview ?
                    jsx(CompressOutlined, { className: "item", onClick: function () { return setPreview(false); } }, void 0)
                    :
                        jsx(ExpandOutlined, { className: "item", onClick: function () { return setPreview(true); } }, void 0) }), void 0)] }, void 0));
};

var md = new MarkdownIt({
    breaks: true,
    highlight: function (code, language) {
        if (language && hljs.getLanguage(language)) {
            try {
                return "<pre><code class=\"hljs language-" + language + "\">" +
                    hljs.highlight(code, { language: language }).value +
                    '</code></pre>';
            }
            catch (__) { }
        }
        return '<pre class="hljs"><code>' + md.utils.escapeHtml(code) + '</code></pre>';
    }
});

var IKeyCodeMap;
(function (IKeyCodeMap) {
    IKeyCodeMap[IKeyCodeMap["ctrlZ"] = 90] = "ctrlZ";
    IKeyCodeMap[IKeyCodeMap["ctrlY"] = 89] = "ctrlY";
    IKeyCodeMap[IKeyCodeMap["ctrlB"] = 66] = "ctrlB";
    IKeyCodeMap[IKeyCodeMap["ctrlI"] = 73] = "ctrlI";
    IKeyCodeMap[IKeyCodeMap["shift9"] = 57] = "shift9";
    IKeyCodeMap[IKeyCodeMap["shiftArrayBracket"] = 219] = "shiftArrayBracket";
    IKeyCodeMap[IKeyCodeMap["oneMark"] = 222] = "oneMark";
    IKeyCodeMap[IKeyCodeMap["tab"] = 9] = "tab";
    IKeyCodeMap[IKeyCodeMap["enter"] = 13] = "enter";
})(IKeyCodeMap || (IKeyCodeMap = {}));

var scrolling = 0;
var scrollTimer;
var renderTimer;
var history = { value: '', pre: null, next: null, selectionStart: 0, selectionEnd: 0 };
var historyTimer;
var MarkdownEditor = function (_a) {
    var _b = _a.initialValue, initialValue = _b === void 0 ? '' : _b, _c = _a.onChange, onChange = _c === void 0 ? function () { } : _c, _d = _a.tabSpace, tabSpace = _d === void 0 ? 4 : _d;
    var _e = useState(''), value = _e[0], setValue = _e[1];
    var _f = useState(''), htmlString = _f[0], setHtmlString = _f[1];
    var _g = useState(['']), line = _g[0], setLine = _g[1];
    var _h = useState(true), loading = _h[0], setLoading = _h[1];
    var _j = useState(false), preview = _j[0], setPreview = _j[1];
    var lineNode = useRef(null);
    var editorNode = useRef(null);
    var showNode = useRef(null);
    var handleChange = useCallback(function (e) {
        onChange(e.target.value, md.render(e.target.value));
        wrapSetValue(e.target.value);
    }, []);
    var haneleScroll = useCallback(function (e) {
        var target = e.target;
        var scale = getScale(target);
        lineNode.current.scrollTop = target.scrollTop;
        if (target.nodeName === 'TEXTAREA') {
            if (scrolling === 0)
                scrolling = 1;
            else if (scrolling === 2)
                return;
            driveScroll(scale, showNode.current);
        }
        else {
            if (scrolling === 0)
                scrolling = 2;
            else if (scrolling === 1)
                return;
            driveScroll(scale, editorNode.current);
        }
    }, []);
    var driveScroll = useCallback(function (scale, el) {
        var scrollHeight = el.scrollHeight, clientHeight = el.clientHeight;
        el.scrollTop = (scrollHeight - clientHeight) * scale;
        if (scrollTimer)
            clearTimeout(scrollTimer);
        scrollTimer = setTimeout(function () {
            scrolling = 0;
            clearTimeout(scrollTimer);
        }, 200);
    }, []);
    var getScale = useCallback(function (node) {
        var scrollHeight = node.scrollHeight, scrollTop = node.scrollTop, clientHeight = node.clientHeight;
        return scrollTop / (scrollHeight - clientHeight);
    }, []);
    var handleKeyDown = useCallback(function (e) {
        var keyCode = e.keyCode, ctrlKey = e.ctrlKey; e.altKey; var shiftKey = e.shiftKey;
        var el = e.target;
        // console.log(keyCode)
        if (ctrlKey) {
            switch (keyCode) {
                case IKeyCodeMap.ctrlZ:
                    if (!history.pre)
                        return e.preventDefault();
                    var _a = history.pre, value = _a.value, selectionStart = _a.selectionStart, selectionEnd = _a.selectionEnd;
                    setValue(value);
                    history = history.pre;
                    setSelectionRange(editorNode.current, selectionStart, selectionEnd);
                    e.preventDefault();
                    break;
                case IKeyCodeMap.ctrlY:
                    if (!history.next)
                        return e.preventDefault();
                    var _b = history.next, value = _b.value, selectionStart = _b.selectionStart, selectionEnd = _b.selectionEnd;
                    setValue(value);
                    history = history.next;
                    setSelectionRange(editorNode.current, selectionStart, selectionEnd);
                    e.preventDefault();
                    break;
                case IKeyCodeMap.ctrlB:
                    handleText(editorNode.current, '**', '', wrapSetValue);
                    e.preventDefault();
                    break;
                case IKeyCodeMap.ctrlI:
                    handleText(editorNode.current, '*', '', wrapSetValue);
                    e.preventDefault();
                    break;
            }
        }
        else if (shiftKey) {
            switch (keyCode) {
                case IKeyCodeMap.tab:
                    cancelTabSpace(el, tabSpace, wrapSetValue);
                    e.preventDefault();
                    break;
                case IKeyCodeMap.shift9:
                    autoComplementBracket(el, wrapSetValue, ['(', ')']);
                    e.preventDefault();
                    break;
                case IKeyCodeMap.shiftArrayBracket:
                    autoComplementBracket(el, wrapSetValue, ['{', '}']);
                    e.preventDefault();
                    break;
                case IKeyCodeMap.oneMark:
                    autoComplementBracket(el, wrapSetValue, ['\"', '\"']);
                    e.preventDefault();
                    break;
            }
        }
        else {
            switch (keyCode) {
                case IKeyCodeMap.tab:
                    addList(el, ' '.repeat(tabSpace), wrapSetValue, 2);
                    e.preventDefault();
                    break;
                case IKeyCodeMap.enter:
                    clickEnter(el, wrapSetValue, tabSpace);
                    e.preventDefault();
                    break;
                case IKeyCodeMap.shiftArrayBracket:
                    autoComplementBracket(el, wrapSetValue, ['[', ']']);
                    e.preventDefault();
                    break;
                case IKeyCodeMap.oneMark:
                    autoComplementBracket(el, wrapSetValue, ['\'', '\'']);
                    e.preventDefault();
                    break;
            }
        }
    }, []);
    var wrapSetValue = function (value, selectionStart, selectionEnd) {
        if (selectionStart === void 0) { selectionStart = null; }
        if (selectionEnd === void 0) { selectionEnd = null; }
        setValue(value);
        setHistory(selectionStart, selectionEnd);
    };
    var setHistory = function (selectionStart, selectionEnd) {
        var _a = getCursorPosition(editorNode.current), start = _a[0], end = _a[1];
        if (historyTimer)
            clearTimeout(historyTimer);
        historyTimer = setTimeout(function () {
            history.next = {
                value: editorNode.current.value,
                pre: history,
                next: null,
                selectionStart: selectionStart ? selectionStart : start,
                selectionEnd: selectionEnd ? selectionEnd : end
            };
            history = history.next;
            clearTimeout(historyTimer);
        }, 1000);
    };
    useEffect(function () {
        if (renderTimer)
            clearTimeout(renderTimer);
        setLine(editorNode.current.value.split('\n'));
        renderTimer = setTimeout(function () {
            setHtmlString(md.render(value));
            clearTimeout(renderTimer);
        }, 200);
    }, [value]);
    useEffect(function () {
        history.value = initialValue;
        setValue(initialValue);
        var len = initialValue.length;
        setSelectionRange(editorNode.current, len, len);
    }, []);
    return (jsxs(MarkdownEditContainer, { children: [jsx(NavBar, { preview: preview, setPreview: setPreview, setValue: wrapSetValue, editorElement: editorNode.current, setLoading: setLoading }, void 0),
            jsx(Spin, __assign({ size: "large", spinning: loading, wrapperClassName: "spining" }, { children: jsxs("div", __assign({ className: "markdown-main" }, { children: [jsx("div", __assign({ className: "line-container", ref: lineNode }, { children: line.map(function (item, index) {
                                return (jsx("p", __assign({ className: "line-num" }, { children: index + 1 }), index));
                            }) }), void 0),
                        jsx("textarea", { value: value, className: "markdown-editor " + (preview ? 'hide' : ''), ref: editorNode, onChange: handleChange, onScroll: haneleScroll, onKeyDown: handleKeyDown }, void 0),
                        jsx("div", { className: "markdown-preview markdown-body " + (preview ? 'prev' : ''), id: "write", ref: showNode, onScroll: haneleScroll, dangerouslySetInnerHTML: { __html: htmlString } }, void 0)] }), void 0) }), void 0)] }, void 0));
};
var MarkdownEditor$1 = memo(MarkdownEditor);

export default MarkdownEditor$1;
